import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)
import java.lang.NullPointerException;
import java.util.*;

/**
 * A thing that can move around with a certain speed.
 * 
 * @author Poul Henriksen
 */
public abstract class MovingThing extends Actor
{
    private Vector speed = new Vector();
    
    private double x = 0;
    private double y = 0;
    
    public MovingThing()
    {
    }
    
    /**
     * Create new thing initialised with given speed.
     */
    public MovingThing(Vector speed)
    {
        this.speed = speed;
    }
    
    /**
     * Move forward one step.
     */
    public void move() {
        x = x + speed.getX();
        y = y + speed.getY();
        
        setLocation(x, y);
    }
    
    public void setLocation(double x, double y) {
        this.x = x;
        this.y = y;
        super.setLocation((int) x, (int) y);
    }
    
    public void setLocation(int x, int y) {
        setLocation((double) x, (double) y);
    }

    /**
     * Increase the speed with the given vector.
     */
    public void increaseSpeed(Vector s) {
        speed.add(s);
    }
    /**
     * Return the current speed.
     */
    public Vector getSpeed() {
        return speed;
    }
    
    public TankField getField()
    {
        TankField   f = (TankField) getWorld();
        return f;
    }
    
    public void turnTo()
    {
       try
       {
       int turnAngle = 0;
       int deltaX = Greenfoot.getMouseInfo().getX() - this.getX();
       int deltaY = Greenfoot.getMouseInfo().getY() - this.getY();
       turnAngle = (int) (180 * Math.atan2(deltaY, deltaX) / Math.PI);
       setRotation(turnAngle);
       }catch (NullPointerException e)
       {
                
       }
       
       }
    
    public void turnTo(Actor a)
    {
       int turnAngle = 0;
       int deltaX = a.getX() - this.getX();
       int deltaY = a.getY() - this.getY();
       turnAngle = (int) (180 * Math.atan2(deltaY, deltaX) / Math.PI);
       setRotation(turnAngle) ;
    }

    /** this was from mover **/
    
        private static final double WALKING_SPEED = 5.0;
    
    /**
     * Turn 90 degrees to the right (clockwise).
     */
    public void turn()
    {
        turn(90);
    }
    

    /**
     * Turn 'angle' degrees towards the right (clockwise).
     */
    public void turn(int angle)
    {
        setRotation(getRotation() + angle);
    }
    

    /**
     * Move a bit forward in the current direction.
     */
    public void move2()
    {
        move2(WALKING_SPEED);
    }

    
    /**
     * Move a specified distance forward in the current direction.
     */
    public void move2(double distance)
    {
        double angle = Math.toRadians( getRotation() );
        int x = (int) Math.round(getX() + Math.cos(angle) * distance);
        int y = (int) Math.round(getY() + Math.sin(angle) * distance);
        
        setLocation(x, y);
    }

    
    /**
     * Test if we are close to one of the edges of the world. Return true is we are.
     */
    public boolean atWorldEdge()
    {
        if(getX() < 20 || getX() > getWorld().getWidth() - 20)
            return true;
        if(getY() < 20 || getY() > getWorld().getHeight() - 20)
            return true;
        else
            return false;
    }
    
}
